
# q-expansion of newform 180.6.a.b, downloaded from the LMFDB on 01 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 180
weight = 6
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-25], [56], [-156], [350], [-786], [740], [-2376], [-2574], [-4576], [-12202], [10230], [-16084], [-864], [17658], [-48684], [-33778], [3524], [-38280], [-79702], [99248], [22284], [-94650], [9122], [71562], [-31816], [175092], [41918], [149262], [60176], [167628], [115542], [163196], [-299238], [288440], [430382], [260420], [465672], [-527454], [-6084], [109670], [-285984], [-263806], [-297174], [-945592], [178820], [-343600], [1324380], [-1399594], [-1028490], [517392], [-1571662], [-1267788], [1271742], [-1210296], [-2294526], [1277264], [1069382], [739974], [1338908], [-836598], [1558388], [-108984], [-1464166], [997074], [1911452], [375698], [4418052], [2833646], [3630942], [-3078024], [-165184], [-587674], [691628], [4058352], [-3440406], [3383198], [4060014], [3531386], [-995892], [-251050], [-5883408], [1834034], [-667912], [-1271484], [-5046018], [1665770], [-427326], [-5197312], [-166548], [2635296], [-8444968], [5488836], [-6419836], [-7212264], [10377618], [-6396522], [-680692], [-1010962], [-5399068], [-13367262], [-1823700], [7081254], [8925260], [4716482], [-9584556], [134574], [6512904], [10762298], [-1704592], [12348758], [-4164234], [11061404], [7699352], [-2203074], [17733284], [-18937368], [17769474], [16658652], [1612358], [2879330], [6863754], [-11689644], [-1651612], [7038354], [-14200906], [2101872], [4956872], [810926], [18943220], [782376], [-26306128], [-20770906], [-13557402], [29120642], [8492778], [10717652], [-15030222], [-30926346], [12589820], [-22459206], [-16977592], [2217828], [-5022898], [-31157544], [-3887098], [-16699962], [12008780], [8455248], [-34629826], [-33658290], [-28660588], [-3462216], [31179308], [38915472], [-5388712], [-3510498], [-13354102], [-32436510], [27920652], [47230566], [6854840], [40376292], [6301998], [-50885448], [-6058912], [22866134], [-10191694], [-49870086], [-37112748], [30465422], [3934008], [6941162], [-7953616], [-44807418], [57745964], [-34581750], [67054520], [-41413186], [-3545776], [-14440116], [74078198], [63542934], [-44772960], [47699610], [35870894], [-21281212], [-42972022], [40414176], [-77394814], [-49464204], [17612036], [-30071694], [80614812], [19296054], [-72339598], [3951950], [9422334], [-27090360], [-44251326], [16039184], [-37248058], [9155426], [-41739996], [7544274], [-57767776], [-19336356], [-88699434], [-10661092], [20474258], [-87616230], [-100299544], [-47869116], [-63328776], [-15598390], [-113622208], [79701294], [2747928], [-28878606], [-40668778], [32509304], [-126849090], [-123588736], [132121260], [82432454], [132671238], [73250208], [-6836968], [-64682940], [85199486], [-39557500], [-82513600], [16236822], [-128604532], [25877376], [83123090], [41308314], [41138868], [-17897832], [31965036], [75419468], [143835032], [-17730082], [119132526], [20391432], [14305520], [63113964], [24909980], [-38462208], [60262478], [-8205762], [-120249432], [191483498], [-7799358], [42169308], [-47332858], [87151676], [62699850], [85839194], [27204656], [-95504772], [-194905738], [158648750], [-213603426], [-91936012], [132757218], [189465702], [169233596], [-109674582], [-218270434], [-243504556], [-199058566], [-21872704], [-236096590], [74667272], [141070116], [-238226290], [11259818], [-43758852], [268913616], [-179571448], [-91393224], [67665782], [-51827476], [-121215408], [-250500526], [44647194], [125424536], [54439326], [11184354], [-269627508], [-244133850], [112869924], [-146773282], [165442674], [178314080], [167904570], [-324494124], [-118052284], [246529898], [-164971902], [79737008], [304455564], [-265946644], [190248098], [192582516], [-14499970], [226810536], [-309496138], [24805536], [-218076390], [15759390], [20523140], [181322376], [267867338], [56493564], [306448992], [-93690430], [-270171666], [-190136380], [-339710662], [121508466], [89561744], [182243958], [-91059406], [176183444], [-359050468], [-418122960], [-399134838], [452212094], [203637714], [233605664], [-336416100], [-421421092], [25470084], [157793198], [-64992546], [24873482], [170139008], [-328023706], [488993190], [-289386198], [453957032], [-130148814], [-379083252], [-125076778], [222493004], [84473520], [-116946438], [-395060620], [-314314774], [-131054526], [-35931712], [-133299130], [-148995642], [-206264352], [-151985148], [444672270], [223852248], [-47621002], [-10561194], [-401797440], [-318749772], [463997540], [-464151670], [539754018], [-468507112], [316748090], [-515195220], [-507455524], [-386896704], [-30220422], [-311143528], [246869390], [177219036], [-164207808], [-30459742], [-21205170], [463428890], [440107026], [-328891626], [17934536], [-211995042], [-216087436], [-307734552], [-208484752], [-31776346], [112796732], [324935952], [-70731646], [-535626774], [-204451260], [424727252], [-632577240], [639225722], [590501120], [705047286], [-430616068], [115582458], [467370638], [39030270], [-180713632], [-140799930], [-142201014], [-433328056], [428320958], [88563342], [297977108], [-641675796], [-617243182], [346364826], [664523556], [650197460], [509981450], [766351650], [265169856], [-454521928], [31087278], [583340088], [-578229966], [-87282346], [-432626784], [-301460652], [-107390806], [-277341438], [370631052], [356171142], [451373420], [96461160]]
